"""URL Configuration."""
from django.conf import settings
from django.http import HttpResponse
from django.urls import include, path


def build_urlpatterns(*paths):
    """
    Generate urlpatterns, adding URL_PREFIX, health checks, prometheus urls, and optional static files urls.

    Valid input is what would normally be assigned in urlpatterns

    ```
    paths = [
        path('', include('{project}.apps.{app}.urls'))
    ]
    ```
    """
    urlpatterns = [path(settings.URL_PREFIX, include(list(paths))), ]

    # Paths that check services health
    urlpatterns += [
        path('healthz', lambda request: HttpResponse(status=200), name='health-check'),
        path('readyz', lambda request: HttpResponse(status=200), name='ready-check'),
    ]

    # Prometheus paths
    urlpatterns += [path('', include('django_prometheus.urls')), ]

    # Include static file urls in development mode
    if not settings.AWS_STORAGE:
        from django.contrib.staticfiles.urls import staticfiles_urlpatterns
        urlpatterns += staticfiles_urlpatterns()

    return urlpatterns



import sys
import traceback
import logging
from collections import OrderedDict
from django.conf import settings
from rest_framework.views import exception_handler
from rest_framework.response import Response
from rest_framework import status

from t3.util import get_response_type


log = logging.getLogger(__name__)


def enveloped_exception_handler(exc, context):
    # Call REST framework's default exception handler first,
    # to get the standard error response.
    exc_info = sys.exc_info()
    response = exception_handler(exc, context)
    traceback_list = ''.join(traceback.format_exc()).split('\n')

    # Gather exception details, such as trace info
    exception_details = OrderedDict([
        ('exception_class', str(exc_info[0])),
        ('exception_message', str(exc_info[1])),
        ('exception_traceback', traceback_list),
    ])

    # Now add the HTTP status code to the response.
    if not response:
        response = Response({}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    # Build response data
    response.data = OrderedDict([
        ('status', get_response_type(response.status_code)),
        ('code', response.status_code),
    ])

    # If debug mode is on, return exception details
    if settings.DEBUG:
        response.data['data'] = exception_details

        # If 500, log exception details
    if response.status_code == 500:
        log.error(OrderedDict([
            ('status', get_response_type(response.status_code)),
            ('code', response.status_code),
            ('data', dict(exception_details)),
        ]))

    # If debug is on, return trace info
    if settings.DEBUG:
        response.data['data'] = exception_details

    return response



def get_response_type(code):
    if 100 <= code <= 199:
        return 'informational'
    if 200 <= code <= 299:
        return 'success'
    if 300 <= code <= 399:
        return 'redirect'
    if 400 <= code <= 499:
        return 'client_error'
    if 500 <= code <= 599:
        return 'server_error'
    return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

import os
from setuptools import setup, find_packages
import versioneer


here = os.path.abspath(os.path.dirname(__file__))

readme_file = os.path.join(here, 'README.md')
try:
    from m2r import parse_from_file
    readme = parse_from_file(readme_file)
except ImportError:
    # m2r may not be installed in user environment
    with open(readme_file) as f:
        readme = f.read()

# get the dependencies and installs
with open(os.path.join(here, 'requirements.txt'), encoding='utf-8') as f:
    all_reqs = f.read().split('\n')

install_requires = [x.strip() for x in all_reqs if 'git+' not in x]
dependency_links = [x.strip().replace('git+', '') for x in all_reqs if x.startswith('git+')]
tests_require = []

setup(
    author="Travis Krause",
    author_email='travis.krause@t-3.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.6',
    ],
    description="Boilerplate to quickly setup a Django Rest Framework Microservice for T3",
    long_description=readme,
    install_requires=install_requires,
    dependency_links=dependency_links,
    include_package_data=True,
    keywords='t3 t3-python-core',
    name='t3-core',
    packages=find_packages('./src'),
    package_dir={'': 'src'},
    test_suite='tests',
    tests_require=tests_require,
    url='https://github.com/nwcell/t3_django_microservice',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    # zip_safe=False,
)
